/*
 * Copyright (C) 2008-2009 the original author or authors
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *         http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.google.code.liquidform;

/**
 * Base class to be inherited by clauses that can be chained. Holds a reference
 * to the previous clause and knows how to construct {@link #toString()}
 * representations of a complete query.
 * 
 * 
 */
/* default */abstract class AbstractClause {

	protected final AbstractClause previousClause;

	protected AbstractClause(AbstractClause previousClause) {
		this.previousClause = previousClause;
	}

	/**
	 * Return a String representation of this piece of the query. Note that a
	 * valid JPA query must contain at least a <tt>SELECT</tt> and a
	 * <tt>FROM</tt> part.
	 */
	@Override
	public final String toString() {
		return toString(new StringBuilder()).toString();
	}

	protected final StringBuilder toString(StringBuilder buffer) {
		if (previousClause != null) {
			previousClause.toString(buffer);
		}
		this.addTo(buffer);
		return buffer;
	}

	/**
	 * Add this clause representation to the passed in buffer. Implementations
	 * typically return the passed in buffer.
	 * 
	 * @param buffer
	 *            a non <tt>null</tt> StringBuilder, maybe already containing
	 *            the string representation of other clauses.
	 * @return the same buffer
	 */
	/* default */abstract StringBuilder addTo(StringBuilder buffer);

}
