/*
 * Copyright (C) 2008-2009 the original author or authors
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *         http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.google.code.liquidform.conditions;

import com.google.code.liquidform.internal.AliasFactory;

/**
 * An expression that has a left and right operand. Operands may be
 * identification variables or association paths, or other
 * {@link ConditionalExpression}s (as in <tt>x AND y</tt>).
 * 
 * @param <L>
 *            the type of the left operand
 * @param <R>
 *            the type of the right operand
 */
public class BinaryExpression<L, R> extends ConditionalExpression {

	private final L lhs;

	private final R rhs;

	private Operator operator;

	public BinaryExpression(L lhs, Operator operator, R rhs) {
		this.lhs = lhs;
		this.operator = operator;
		this.rhs = rhs;
	}

	@Override
	public String getTargetRepresentation() {
		StringBuilder sb = new StringBuilder();

		// lhs
		sb.append(AliasFactory.safeStringForm(lhs));

		// op
		sb.append(operator.getStringRepresentation());

		// rhs
		sb.append(AliasFactory.safeStringForm(rhs));

		return sb.toString();
	}

}
