/*
 * Copyright (C) 2008-2009 the original author or authors
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *         http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.google.code.liquidform.conditions;

import com.google.code.liquidform.LiquidForm;
import com.google.code.liquidform.internal.managedobjects.FrameworkManagedObject;

/**
 * Represents a filter expression that will eventually resolve as a boolean.
 * {@link ConditionalExpression}s can be combined together using
 * {@link #and(ConditionalExpression)} or {@link #or(ConditionalExpression)},
 * and negated using {@link LiquidForm#not(ConditionalExpression)}.
 * 
 */
public abstract class ConditionalExpression extends FrameworkManagedObject {

	/* default */static enum Type implements Operator {
		AND(" and "), OR(" or ");

		private Type(String stringForm) {
			this.stringForm = stringForm;
		}

		private final String stringForm;

		public String getStringRepresentation() {
			return stringForm;
		}
	}

	/**
	 * Combine <i>this</i> expression with another, using the boolean operator
	 * <tt>AND</tt>.
	 */
	public ConditionalExpression and(ConditionalExpression rhs) {
		return new BinaryExpression<ConditionalExpression, ConditionalExpression>(
				this, Type.AND, rhs);
	}

	/**
	 * Combine <i>this</i> expression with another, using the boolean operator
	 * <tt>OR</tt>.
	 */
	public ConditionalExpression or(ConditionalExpression rhs) {
		return new BinaryExpression<ConditionalExpression, ConditionalExpression>(
				this, Type.OR, rhs);
	}

}
