/*
 * Copyright (C) 2008-2009 the original author or authors
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *         http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.google.code.liquidform.conditions;

import com.google.code.liquidform.internal.AliasFactory;

/**
 * An expression that has 3 operands. Examples include ones with the
 * {@link LikeOperator LIKE} or {@link BetweenOperator BETWEEN} operators.
 * 
 * @param <T>
 *            The type of the left hand side operand
 * @param <U>
 *            The type of the middle, or first right hand side operand
 * @param <V>
 *            The type of the last, or second right hand side operand
 */
public class TernaryExpression<T, U, V> extends ConditionalExpression {

	private final T lhs;

	private final U rhs1;

	private final V rhs2;

	private final TernaryOperator operator;

	public TernaryExpression(T lhs, TernaryOperator operator1, U rhs1, V rhs2) {
		this.lhs = lhs;
		this.operator = operator1;
		this.rhs1 = rhs1;
		this.rhs2 = rhs2;
	}

	@Override
	public String getTargetRepresentation() {
		StringBuilder sb = new StringBuilder();

		// lhs
		sb.append(AliasFactory.safeStringForm(lhs));

		// op1
		sb.append(operator.getStringRepresentation());

		// rhs1
		sb.append(AliasFactory.safeStringForm(rhs1));

		// op2
		sb.append(operator.getSecondStringRepresentation());

		// rhs2
		sb.append(AliasFactory.safeStringForm(rhs2));
		return sb.toString();
	}

}
