/*
 * Copyright (C) 2008-2009 the original author or authors
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *         http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.google.code.liquidform.conditions;

import com.google.code.liquidform.internal.AliasFactory;

/**
 * An expression that involves a single operand, prefixed by an {@link Operator}
 * .
 * 
 * @param <T>
 *            the type of the operand
 */
public class UnaryExpression<T> extends ConditionalExpression {

	private final T operand;

	private final Operator operator;

	public UnaryExpression(T operand, Operator operator) {
		this.operand = operand;
		this.operator = operator;
	}

	@Override
	public String getTargetRepresentation() {
		StringBuilder sb = new StringBuilder();

		// op
		sb.append(operator.getStringRepresentation());
		// lhs
		sb.append(AliasFactory.safeStringForm(operand));

		return sb.toString();

	}

}
